use lazy_static::lazy_static;
use regex::Regex;

use crate::Client;

// this lazy static stuff is so not to recompile the re every time
// unwrap is ok here because compiler will catch it if the string is bad
lazy_static! {
	static ref RE: Regex = Regex::new(r"^#!(?P<action>\w+)(?: (?P<text>.*))?$").unwrap();
}

/// type of log action
#[derive(Debug)]
pub enum Action {
	/// specify client type
	Client(Client),

	/// insert a comment in the log
	Comment(String),

	/// delay replay
	/// duration is number of seconds or 1 if no seconds specified or if duration can't be parsed
	Delay(u64),

	/// log line
	/// duration is amount of time to sleep before printing the line
	Line(String),

	/// insert a marker
	Mark,

	/// No match
	None,
}

impl From<&str> for Action {
	fn from(line: &str) -> Self {
		if line.starts_with("#!") {
			let caps = RE.captures(&line).unwrap();

			let action = if let Some(action) = caps.name("action") { action.as_str() } else { "" };

			let text = if let Some(text) = caps.name("text") {
				text.as_str().to_owned()
			} else {
				String::new()
			};

			match action {
				"client" => Self::Client(Client::from(&text)),
				"delay" => Self::Delay(text.parse::<u64>().unwrap_or(1)),
				"mark" => Self::Mark,
				_ => Self::None,
			}
		} else if line.starts_with("##") {
			Self::Comment(line.to_owned())
		} else {
			Self::Line(String::new())
		}
	}
}
